@@ -74,7 +74,7 @@ def __virtual__():
     '''
     if not HAS_BOTO:
         return False
-    __utils__['boto.assign_funcs'](__name__, 'elasticache')
+    __utils__['boto.assign_funcs'](__name__, 'elasticache', pack=__salt__)
     return True
 
 
