@@ -9,9 +9,11 @@ Manage Dell DRAC.
 # Import python libs
 from __future__ import absolute_import
 import logging
+import os
 import re
 
 # Import Salt libs
+from salt.exceptions import CommandExecutionError
 import salt.utils
 
 # Import 3rd-party libs
@@ -21,6 +23,8 @@ from salt.ext.six.moves import map
 
 log = logging.getLogger(__name__)
 
+__proxyenabled__ = ['fx2']
+
 try:
     run_all = __salt__['cmd.run_all']
 except NameError:
@@ -88,7 +92,8 @@ def __execute_cmd(command, host=None,
                                                          admin_username,
                                                          admin_password,
                                                          command,
-                                                         modswitch))
+                                                         modswitch),
+        output_loglevel='quiet')
 
     if cmd['retcode'] != 0:
         log.warning('racadm return an exit code \'{0}\'.'
@@ -121,7 +126,8 @@ def __execute_ret(command, host=None,
                                                          admin_username,
                                                          admin_password,
                                                          command,
-                                                         modswitch))
+                                                         modswitch),
+        output_loglevel='quiet')
 
     if cmd['retcode'] != 0:
         log.warning('racadm return an exit code \'{0}\'.'
@@ -131,8 +137,13 @@ def __execute_ret(command, host=None,
         for l in cmd['stdout'].splitlines():
             if l.startswith('Security Alert'):
                 continue
+            if l.startswith('RAC1168:'):
+                break
+            if l.startswith('RAC1169:'):
+                break
             if l.startswith('Continuing execution'):
                 continue
+
             if len(l.strip()) == 0:
                 continue
             fmtlines.append(l)
@@ -155,7 +166,8 @@ def get_dns_dracname(host=None,
 
 def set_dns_dracname(name,
                      host=None,
-                     admin_username=None, admin_password=None):
+                     admin_username=None,
+                     admin_password=None):
 
     ret = __execute_ret('set iDRAC.NIC.DNSRacName {0}'.format(name),
                         host=host,
@@ -189,7 +201,7 @@ def system_info(host=None,
     return __parse_drac(cmd['stdout'])
 
 
-def set_niccfg(ip=None, subnet=None, gateway=None, dhcp=False,
+def set_niccfg(ip=None, netmask=None, gateway=None, dhcp=False,
                host=None,
                admin_username=None,
                admin_password=None,
@@ -200,7 +212,7 @@ def set_niccfg(ip=None, subnet=None, gateway=None, dhcp=False,
     if dhcp:
         cmdstr += '-d '
     else:
-        cmdstr += '-s ' + ip + ' ' + subnet + ' ' + gateway
+        cmdstr += '-s ' + ip + ' ' + netmask + ' ' + gateway
 
     return __execute_cmd(cmdstr, host=host,
                          admin_username=admin_username,
@@ -249,10 +261,10 @@ def network_info(host=None,
         cmd['stdout'] = 'Problem getting switch inventory'
         return cmd
 
-    if module not in inv.get('switch'):
+    if module not in inv.get('switch') and module not in inv.get('server'):
         cmd = {}
         cmd['retcode'] = -1
-        cmd['stdout'] = 'No switch {0} found.'.format(module)
+        cmd['stdout'] = 'No module {0} found.'.format(module)
         return cmd
 
     cmd = __execute_ret('getniccfg', host=host,
@@ -457,7 +469,11 @@ def change_password(username, password, uid=None, host=None,
     be necessary if one is not sure which user slot contains the one you want.
     Many late-model Dell chassis have 'root' as UID 1, so if you can depend
     on that then setting the password is much quicker.
+    Raises an error if the supplied password is greater than 20 chars.
     '''
+    if len(password) > 20:
+        raise CommandExecutionError('Supplied password should be 20 characters or less')
+
     if uid is None:
         user = list_users(host=host, admin_username=admin_username,
                           admin_password=admin_password, module=module)
@@ -1309,8 +1325,55 @@ def get_general(cfg_sec, cfg_var, host=None,
         return ret
 
 
+def idrac_general(blade_name, command, idrac_password=None,
+                  host=None,
+                  admin_username=None, admin_password=None):
+    '''
+    Run a generic racadm command against a particular
+    blade in a chassis.  Blades are usually named things like
+    'server-1', 'server-2', etc.  If the iDRAC has a different
+    password than the CMC, then you can pass it with the
+    idrac_password kwarg.
+
+    :param blade_name: Name of the blade to run the command on
+    :param command: Command like to pass to racadm
+    :param idrac_password: Password for the iDRAC if different from the CMC
+    :param host: Chassis hostname
+    :param admin_username: CMC username
+    :param admin_password: CMC password
+    :return: stdout if the retcode is 0, otherwise a standard cmd.run_all dictionary
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt fx2 chassis.cmd idrac_general server-1 'get BIOS.SysProfileSettings'
+
+    '''
+
+    module_network = network_info(host, admin_username,
+                                  admin_password, blade_name)
+
+    if idrac_password is not None:
+        password = idrac_password
+    else:
+        password = admin_password
+
+    idrac_ip = module_network['Network']['IP Address']
+
+    ret = __execute_ret(command, host=idrac_ip,
+                        admin_username='root',
+                        admin_password=password)
+
+    if ret['retcode'] == 0:
+        return ret['stdout']
+    else:
+        return ret
+
+
 def bare_rac_cmd(cmd, host=None,
                 admin_username=None, admin_password=None):
+
     ret = __execute_ret('{0}'.format(cmd),
                         host=host,
                         admin_username=admin_username,
@@ -1320,3 +1383,100 @@ def bare_rac_cmd(cmd, host=None,
         return ret['stdout']
     else:
         return ret
+
+
+def _update_firmware(cmd,
+                     host=None,
+                     admin_username=None,
+                     admin_password=None):
+
+    if not admin_username:
+        admin_username = __pillar__['proxy']['admin_username']
+    if not admin_username:
+        admin_password = __pillar__['proxy']['admin_password']
+
+    ret = __execute_ret(cmd,
+                        host=host,
+                        admin_username=admin_username,
+                        admin_password=admin_password)
+
+    if ret['retcode'] == 0:
+        return ret['stdout']
+    else:
+        return ret
+
+
+def update_firmware(filename,
+                    host=None,
+                    admin_username=None,
+                    admin_password=None):
+    '''
+    Updates firmware using local firmware file
+
+    .. code-block:: bash
+
+         salt dell dracr.update_firmware firmware.exe
+
+    This executes the following command on your FX2
+    (using username and password stored in the pillar data)
+
+    .. code-block:: bash
+
+         racadm update –f firmware.exe -u user –p pass
+
+    '''
+    if os.path.exists(filename):
+        return _update_firmware('update -f {0}'.format(filename),
+                                host=None,
+                                admin_username=None,
+                                admin_password=None)
+    else:
+        raise CommandExecutionError('Unable to find firmware file {0}'
+                                    .format(filename))
+
+
+def update_firmware_nfs_or_cifs(filename, share,
+                                host=None,
+                                admin_username=None,
+                                admin_password=None):
+    '''
+    Executes the following for CIFS
+    (using username and password stored in the pillar data)
+
+    .. code-block:: bash
+
+         racadm update -f <updatefile> -u user –p pass -l //IP-Address/share
+
+    Or for NFS
+    (using username and password stored in the pillar data)
+
+    .. code-block:: bash
+
+          racadm update -f <updatefile> -u user –p pass -l IP-address:/share
+
+
+    Salt command for CIFS:
+
+    .. code-block:: bash
+
+         salt dell dracr.update_firmware_nfs_or_cifs \
+         firmware.exe //IP-Address/share
+
+
+    Salt command for NFS:
+
+    .. code-block:: bash
+
+         salt dell dracr.update_firmware_nfs_or_cifs \
+         firmware.exe IP-address:/share
+    '''
+    if os.path.exists(filename):
+        return _update_firmware('update -f {0} -l {1}'.format(filename, share),
+                                host=None,
+                                admin_username=None,
+                                admin_password=None)
+    else:
+        raise CommandExecutionError('Unable to find firmware file {0}'
+                                    .format(filename))
+
+# def get_idrac_nic()
